/**
 * 自定义元素
 */
(function(global, doc) {
  var KElement = global.KElement = {};

  /**
   * 根据给定文本和字体大小获得文字渲染后的长度
   */
  var textSize = function(fontSize, text) {
    var span = doc.createElement("span");
    span.style.fontSize = fontSize;
    var result = {};
    result.width = span.offsetWidth;
    result.height = span.offsetWidth; 
    span.style.visibility = "hidden";
    doc.body.appendChild(span);
    span.innerHTML = text;
    result.width = span.offsetWidth - result.width;
    result.height = span.offsetHeight - result.height;
    span.parentNode.removeChild(span);
    return result;
  }

  /**
   * 绘制大括号
   */
  var brace = function(canvas, options) {
    // Default settings
    var defaults = {
      direction: "left", // left, right, up, down
      thickness: 1, // line thickness
      point: 0.5, // position of point
      color: '#212121', // default color of the brace
      sharpness: 1.5 // corner sharpness
    };
    var settings = $.extend(defaults, options || {});
    // make curly brace
    var context = canvas.getContext('2d'),
        horizontal = (settings.direction == "up" || settings.direction == "down"),
        length = (horizontal) ? canvas.width : canvas.height,
        linelength = (length - 60) / 2,
        edge = (horizontal) ? canvas.height / 2 : canvas.width / 2,
        middle = (horizontal) ? canvas.height / 4 : canvas.width / 4;

    if(settings.point <= 1) // percentage
      var pointoffset = ((horizontal) ? canvas.width : canvas.height) * settings.point - 20 - settings.thickness;
    else
      var pointoffset = settings.point - 20 - settings.thickness;

    if(settings.direction == "left" || settings.direction == "up")
      var inner = edge, outer = 0;
    else if(settings.direction == "right" || settings.direction == "down")
      var inner = 0, outer = edge;

    context.lineWidth = settings.thickness;
    context.strokeStyle = settings.color;
    context.beginPath();

    if(settings.direction == "up" || settings.direction == "down") {
      context.moveTo(settings.thickness, inner);
      context.arcTo(settings.thickness, middle, 10 + settings.thickness, middle, middle / settings.sharpness);
      context.lineTo(10 + pointoffset, middle);
      context.arcTo(20 + pointoffset, middle, 20 + pointoffset, outer, middle / settings.sharpness);
      context.arcTo(20 + pointoffset, middle, 30 + pointoffset, middle, middle / settings.sharpness);
      context.lineTo(length - (10 + settings.thickness), middle);
      context.arcTo(length - settings.thickness, middle, length - settings.thickness, inner, middle / settings.sharpness);
      context.lineTo(length - settings.thickness, inner);
    } else {
      context.moveTo(inner, settings.thickness);
      context.arcTo(middle, settings.thickness, middle, 10 + settings.thickness, middle / settings.sharpness);
      context.lineTo(middle, 10 + pointoffset);
      context.arcTo(middle, 20 + pointoffset, outer, 20 + pointoffset, middle / settings.sharpness);
      context.arcTo(middle, 20 + pointoffset, middle, 30 + pointoffset, middle / settings.sharpness);
      context.lineTo(middle, length - (10 + settings.thickness));
      context.arcTo(middle, length - settings.thickness, inner, length - settings.thickness, middle / settings.sharpness);
      context.lineTo(inner, length - settings.thickness);
    }

    context.stroke();

  }

  /*
   * 渲染自定义元素
   * @param element 待渲染的元素对象或CSS选择器
   */
  KElement.render = function(element) {
    // 待渲染的容器集合
    // if (element && element.length > 0) {
      let elementList;
      if (typeof element == 'string') {
        elementList = doc.querySelectorAll(element);
      } else {
        elementList = [element];
      }
      // 渲染元素
      for (let i = 0; i < elementList.length; i++) {
        for (let j = 0; j < this.taskList.length; j++) {
          let {tagName, renderFn} = this.taskList[j];
          let nodeList = elementList[i].querySelectorAll(tagName);
          for (let k = 0; k < nodeList.length; k++) {
            renderFn.call(null, nodeList[k]);
          }
        }
      }
    // }

  }

  /**
   * 编码HTML
   */
  KElement.encodeHTML = function(html) {
    for (let i = 0; i < this.taskList.length; i++) {
      let {encodeFn} = this.taskList[i];
      html = encodeFn.call(null, html);
    }
    return html;
	}

  /**
   * 渲染弧形
   * @param 待渲染的节点
   */
  KElement.renderArc = function(node) {
    var rule = node.getAttribute('data-k-element-rule'),// 渲染规则
      width = rule.split(',')[0],// 待渲染节点的宽
      height = rule.split(',')[1],// 待渲染节点的高
      radius = (Math.pow(width/2, 2) + Math.pow(height, 2)) / (height * 2);// 待渲染弧形的半径
    // console.log(text, width, height, radius);
    // 创建canvas
    var arc = doc.createElement('canvas'),
      context = arc.getContext('2d');
    arc.width = width;
    arc.height = height;
    arc.setAttribute('data-k-element-type', 'arc');
    arc.setAttribute('data-k-element-rule', rule);
    arc.setAttribute('style', node.getAttribute('style'));
    // 绘制弧形
    context.arc(width/2, radius, radius, 0, Math.PI, true);
    context.stroke();
    // 渲染
    node.parentNode.replaceChild(arc, node);
  }

  /**
   * 编码弧形HTML
   */
  KElement.encodeArc = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=arc]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var arc = doc.createElement('k-arc');
      arc.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
      arc.setAttribute('style', node.getAttribute('style'));
      node.parentNode.replaceChild(arc, node);
    }
    return tmp.innerHTML;
  }

  /**
   * 渲染下划线
   */
  KElement.renderUnderlineGap = function(node) {
    var underline = doc.createElement('span');
    underline.setAttribute('data-k-element-type', 'underline');
    underline.setAttribute('contenteditable', false);
    // underline.setAttribute('style', 'display: inline-block; width: 24px; border-bottom: 1px solid #000000');
    underline.innerText = '__________';
    // 渲染
    node.parentNode.replaceChild(underline, node);
  }

  /**
   * 编码下划线HTML
   */
  KElement.encodeUnderlineGap = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=underline]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var underline = doc.createElement('k-underline-gap');
      node.parentNode.replaceChild(underline, node);
    }
    return tmp.innerHTML;
  }

  /**
   * 渲染空
   */
  KElement.renderBlockGap = function(node) {
    var block = doc.createElement('span');
    block.setAttribute('data-k-element-type', 'block');
    block.setAttribute('contenteditable', false);
    block.setAttribute('style', "font-family: '楷体', 'KaiTi', '楷体_GB2312', 'KaiTi_GB2312', Helvetica, Arial, sans-serif;");
    block.innerText = '□';
    // 渲染
    node.parentNode.replaceChild(block, node);
  }

  /**
   * 编码空HTML
   */
  KElement.encodeBlockGap = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=block]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var block = doc.createElement('k-block-gap');
      node.parentNode.replaceChild(block, node);
    }
    return tmp.innerHTML;
  }

  /**
   * 渲染图片
   */
  KElement.renderImg = function(node) {
    var rule = node.getAttribute('data-k-element-rule');// 渲染规则
    var img = doc.createElement('img');
    img.src = rule;
    img.setAttribute('data-k-element-type', 'img');
    img.setAttribute('data-k-element-rule', rule);
    // 渲染
    node.parentNode.replaceChild(img, node);
  }

  /**
   * 编码图片HTML
   */
  KElement.encodeImg = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('img[data-k-element-type=img]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var img = doc.createElement('k-img');
      img.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
      node.parentNode.replaceChild(img, node);
    }
    return tmp.innerHTML;
  }

   /**
    * 渲染等号之上
    */
   KElement.renderAboveEquality = function(node){
     var container =  doc.createElement('span'), content =  doc.createElement('span');
     container.setAttribute('data-k-element-type', 'aboveEquality');

     container.style.lineHeight = '28px';
     container.style.float = 'left';
     content.style.textDecoration = 'underline';
     content.style.paddingBottom  = '2px';
     content.style.borderBottom = '1.6px solid #888';
     content.innerHTML = '&nbsp;&nbsp;&nbsp;&nbsp;' + node.innerHTML + '&nbsp;&nbsp;&nbsp;&nbsp;';
     container.appendChild(content);
     // 渲染
     node.parentNode.replaceChild(container, node);
   }

  /**
   * 编码等号之上
   */
  KElement.encodeAboveEquality = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=aboveEquality]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var aboveEquality = doc.createElement('k-above-equality');
      aboveEquality.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
      aboveEquality.setAttribute('style', node.getAttribute('style'));
      aboveEquality.innerHTML = node.innerHTML;
      node.parentNode.replaceChild(aboveEquality, node);
    }
    return tmp.innerHTML;
  }

   /**
    * 渲染等号之下
    */
   KElement.renderUnderEquality = function(node){
     var container =  doc.createElement('span'), content =  doc.createElement('span');
     container.setAttribute('data-k-element-type', 'underEquality');

     container.style.lineHeight = '37.8px';
     container.style.float = 'left';
     content.style.textDecoration = 'overline';
     content.style.paddingTop  = '3px';
     content.style.borderTop = '1.6px solid #888';
     content.innerHTML = '&nbsp;&nbsp;&nbsp;&nbsp;' + node.innerHTML + '&nbsp;&nbsp;&nbsp;&nbsp;';
     container.appendChild(content);
     // 渲染
     node.parentNode.replaceChild(container, node);
   }

  /**
   * 编码等号之下
   */
  KElement.encodeUnderEquality = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=underEquality]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var underEquality = doc.createElement('k-under-equality');
      underEquality.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
      underEquality.setAttribute('style', node.getAttribute('style'));
      underEquality.innerHTML = node.innerHTML;
      node.parentNode.replaceChild(underEquality, node);
    }
    return tmp.innerHTML;
  }

   /**
    * 渲染右箭头之上
    */
   KElement.renderAboveRightArrow = function(node){
     var rule = node.getAttribute('data-k-element-rule'),// 渲染规则
       width = rule.split(',')[0],// 待渲染节点的宽
       height = rule.split(',')[1];// 待渲染节点的高
     // 创建canvas
     var aboveRightArrow = doc.createElement('canvas'),
       context = aboveRightArrow.getContext('2d');
     aboveRightArrow.width = width;
     aboveRightArrow.height = height;
     aboveRightArrow.setAttribute('data-k-element-type', 'aboveRightArrow');
     aboveRightArrow.setAttribute('data-k-element-rule', rule);
     aboveRightArrow.setAttribute('style', node.getAttribute('style'));
     // 绘制右箭头
     context.moveTo(0, height/2);
     context.lineTo(width, height/2);
     context.moveTo(width, height/2);
     context.lineTo(width-9, height/2 - 6);
     context.moveTo(width, height/2);
     context.lineTo(width-9, height/2 + 6);

     context.stroke();
     // 渲染
     node.parentNode.replaceChild(aboveRightArrow, node);
   }

  /**
   * 编码右箭头之上
   */
  KElement.encodeAboveRightArrow = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=aboveRightArrow]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var aboveRightArrow = doc.createElement('k-above-right-arrow');
      aboveRightArrow.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
      aboveRightArrow.setAttribute('style', node.getAttribute('style'));
      node.parentNode.replaceChild(aboveRightArrow, node);
    }
    return tmp.innerHTML;
  }

   /**
    * 渲染右箭头之下
    */
   KElement.renderUnderRightArrow = function(node){
     var rule = node.getAttribute('data-k-element-rule'),// 渲染规则
       width = rule.split(',')[0],// 待渲染节点的宽
       height = rule.split(',')[1];// 待渲染节点的高
     // 创建canvas
     var underRightArrow = doc.createElement('canvas'),
       context = underRightArrow.getContext('2d');
     underRightArrow.width = width;
     underRightArrow.height = height;
     underRightArrow.setAttribute('data-k-element-type', 'underRightArrow');
     underRightArrow.setAttribute('data-k-element-rule', rule);
     underRightArrow.setAttribute('style', node.getAttribute('style'));
     // 绘制右箭头
     context.moveTo(0, height/2);
     context.lineTo(width, height/2);
     context.moveTo(width, height/2);
     context.lineTo(width-9, height/2 - 6);
     context.moveTo(width, height/2);
     context.lineTo(width-9, height/2 + 6);

     context.stroke();
     // 渲染
     node.parentNode.replaceChild(underRightArrow, node);
   }

  /**
   * 编码右箭头之下
   */
  KElement.encodeUnderRightArrow = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=underRightArrow]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var underRightArrow = doc.createElement('k-under-right-arrow');
      underRightArrow.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
      underRightArrow.setAttribute('style', node.getAttribute('style'));
      node.parentNode.replaceChild(underRightArrow, node);
    }
    return tmp.innerHTML;
  }

  /**
   * 渲染等号公式
   */
  KElement.renderEquality = function(node){
    console.log('renderEquality');
    var outer = doc.querySelector('#k-element-' + node.id);
    var up = doc.querySelector('#up-' + node.id);
    var down = doc.querySelector('#down-' + node.id);

    var upResult, downResult, width, height;
    if(up){
      upResult = textSize(up.style.fontSize, up.innerHTML);
    }
    if(down){
      downResult = textSize(down.style.fontSize, down.innerHTML);
    }
    if(upResult){
      width = upResult.width;
      height = upResult.height;
    }
    if(downResult){
      width = downResult.width;
      height = downResult.height;
    }
    if(upResult && downResult){
      width = upResult.width > downResult.width ? upResult.width : downResult.width;
      height = upResult.height;
    }

    width = width < 14 ?  14 : width;
    height = height < 16 ?  16 : height;

    var canvas = doc.getElementById('center') ? doc.getElementById('center') : doc.createElement('canvas');
    canvas.id = node.id;
    canvas.width = width;
    canvas.height = height;
    canvas.style.display = 'block';
    canvas.setAttribute('data-k-element-type', 'equality');

    var context = canvas.getContext("2d");
    context.beginPath();
    context.lineWidth = 1;
    context.moveTo(0, height*0.3);
    context.lineTo(width, height*0.3);
    context.moveTo(0, height*0.7);
    context.lineTo(width, height*0.7);
    context.stroke();
    context.closePath();

    node.parentNode.replaceChild(canvas, node);
  }

 /**
  * 编码等号公式
  */
 KElement.encodeEquality = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=equality]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var equality = doc.createElement('k-equality');
     equality.id = node.id;
     equality.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(equality , node);
   }
   return tmp.innerHTML;
 }

  /**
   * 渲染左箭头公式
   */
  KElement.renderLeftArrow = function(node){
    var outer = doc.querySelector('#k-element-' + node.id);
    var up = doc.querySelector('#up-' + node.id);
    var down = doc.querySelector('#down-' + node.id);

    var upResult, downResult, width, height;
    if(up){
      upResult = textSize(up.style.fontSize, up.innerHTML);
    }
    if(down){
      downResult = textSize(down.style.fontSize, down.innerHTML);
    }
    if(upResult){
      width = upResult.width;
      height = upResult.height;
    }
    if(downResult){
      width = downResult.width;
      height = downResult.height;
    }
    if(upResult && downResult){
      width = upResult.width > downResult.width ? upResult.width : downResult.width;
      height = upResult.height;
    }

    width = width < 14 ?  14 : width;
    height = height < 16 ?  16 : height;

  	var canvas = doc.getElementById('center') ? doc.getElementById('center') : doc.createElement('canvas');
  	canvas.id = node.id;
  	canvas.width = width;
  	canvas.height = height;
  	canvas.style.display = 'block';
    canvas.setAttribute('data-k-element-type', 'leftArrow');

  	var context = canvas.getContext("2d");
  	context.beginPath();
  	context.moveTo(0, height/2);
  	context.lineTo(width, height/2);
  	context.moveTo(0, height/2);
  	context.lineTo(9, height/2 - 6);
  	context.moveTo(0, height/2);
  	context.lineTo(9, height/2 + 6);
  	context.stroke();
  	context.closePath();

    node.parentNode.replaceChild(canvas, node);
  }

 /**
  * 编码左箭头公式
  */
 KElement.encodeLeftArrow = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=leftArrow]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var leftArrow = doc.createElement('k-left-arrow');
     leftArrow.id = node.id;
     leftArrow.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(leftArrow, node);
   }
   return tmp.innerHTML;
 }

  /**
   * 渲染右箭头公式
   */
  KElement.renderRightArrow = function(node){
    var outer = doc.querySelector('#k-element-' + node.id);
    var up = doc.querySelector('#up-' + node.id);
    var down = doc.querySelector('#down-' + node.id);

    var upResult, downResult, width, height;
    if(up){
      upResult = textSize(up.style.fontSize, up.innerHTML);
    }
    if(down){
      downResult = textSize(down.style.fontSize, down.innerHTML);
    }
    if(upResult){
      width = upResult.width;
      height = upResult.height;
    }
    if(downResult){
      width = downResult.width;
      height = downResult.height;
    }
    if(upResult && downResult){
      width = upResult.width > downResult.width ? upResult.width : downResult.width;
      height = upResult.height;
    }

    width = width < 14 ?  14 : width;
    height = height < 16 ?  16 : height;

  	var canvas = doc.getElementById('center') ? doc.getElementById('center') : doc.createElement('canvas');
  	canvas.id = node.id;
  	canvas.width = width;
  	canvas.height = height;
  	canvas.style.display = 'block';
    canvas.setAttribute('data-k-element-type', 'rightArrow');

  	var context = canvas.getContext("2d");
  	context.beginPath();
  	context.moveTo(0, height/2);
  	context.lineTo(width, height/2);
  	context.moveTo(width, height/2);
  	context.lineTo(width-9, height/2 - 6);
  	context.moveTo(width, height/2);
  	context.lineTo(width-9, height/2 + 6);
  	context.stroke();
  	context.closePath();

    node.parentNode.replaceChild(canvas, node);
  }

 /**
  * 编码右箭头公式
  */
 KElement.encodeRightArrow = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=rightArrow]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var rightArrow = doc.createElement('k-right-arrow');
     rightArrow.id = node.id;
     rightArrow.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(rightArrow, node);
   }
   return tmp.innerHTML;
 }

  /**
   * 渲染正向可逆公式
   */
  KElement.renderPositiveInvertible = function(node){
    var outer = doc.querySelector('#k-element-' + node.id);
    var up = doc.querySelector('#up-' + node.id);
    var down = doc.querySelector('#down-' + node.id);

    var upResult, downResult, width, height;
    if(up){
      upResult = textSize(up.style.fontSize, up.innerHTML);
    }
    if(down){
      downResult = textSize(down.style.fontSize, down.innerHTML);
    }
    if(upResult){
      width = upResult.width;
      height = upResult.height;
    }
    if(downResult){
      width = downResult.width;
      height = downResult.height;
    }
    if(upResult && downResult){
      width = upResult.width > downResult.width ? upResult.width : downResult.width;
      height = upResult.height;
    }

    width = width < 14 ?  14 : width;
    height = height < 16 ?  16 : height;

  	var canvas = doc.getElementById('center') ? doc.getElementById('center') : doc.createElement('canvas');
  	canvas.id = node.id;
  	canvas.width = width;
  	canvas.height = height;
  	canvas.style.display = 'block';
    canvas.setAttribute('data-k-element-type', 'positiveInvertible');

  	var context = canvas.getContext("2d");
  	context.beginPath();
    context.lineWidth = 1;
    context.moveTo(0, height*0.3);
    context.lineTo(width, height*0.3);
    context.moveTo(0, height*0.7);
    context.lineTo(width, height*0.7);
    // 绘箭头
  	context.moveTo(width, height*0.3);
  	context.lineTo(width-9, height*0.3 - 6);
  	context.moveTo(0, height*0.7);
  	context.lineTo(9, height*0.7 + 6);
  	context.stroke();
  	context.closePath();

    node.parentNode.replaceChild(canvas, node);
  }

 /**
  * 编码正向可逆公式
  */
 KElement.encodePositiveInvertible = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=positiveInvertible]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var positiveInvertible = doc.createElement('k-positive-invertible');
     positiveInvertible.id = node.id;
     positiveInvertible.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(positiveInvertible, node);
   }
   return tmp.innerHTML;
 }

  /**
   * 渲染反向可逆公式
   */
  KElement.renderReversedInvertible = function(node){
    var outer = doc.querySelector('#k-element-' + node.id);
    var up = doc.querySelector('#up-' + node.id);
    var down = doc.querySelector('#down-' + node.id);

    var upResult, downResult, width, height;
    if(up){
      upResult = textSize(up.style.fontSize, up.innerHTML);
    }
    if(down){
      downResult = textSize(down.style.fontSize, down.innerHTML);
    }
    if(upResult){
      width = upResult.width;
      height = upResult.height;
    }
    if(downResult){
      width = downResult.width;
      height = downResult.height;
    }
    if(upResult && downResult){
      width = upResult.width > downResult.width ? upResult.width : downResult.width;
      height = upResult.height;
    }

    width = width < 14 ?  14 : width;
    height = height < 16 ?  16 : height;

  	var canvas = doc.getElementById('center') ? doc.getElementById('center') : doc.createElement('canvas');
  	canvas.id = node.id;
  	canvas.width = width;
  	canvas.height = height;
  	canvas.style.display = 'block';
    canvas.setAttribute('data-k-element-type', 'reversedInvertible');

  	var context = canvas.getContext("2d");
  	context.beginPath();
    context.lineWidth = 1;
    context.moveTo(0, height*0.3);
    context.lineTo(width, height*0.3);
    context.moveTo(0, height*0.7);
    context.lineTo(width, height*0.7);
    // 绘箭头
  	context.moveTo(0, height*0.3);
  	context.lineTo(9, height*0.3 - 6);
  	context.moveTo(width, height*0.7);
  	context.lineTo(width-9, height*0.7 + 6);
  	context.stroke();
  	context.closePath();

    node.parentNode.replaceChild(canvas, node);
  }

 /**
  * 编码反向可逆公式
  */
 KElement.encodeReversedInvertible = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=reversedInvertible]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var reversedInvertible = doc.createElement('k-reversed-invertible');
     reversedInvertible.id = node.id;
     reversedInvertible.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(reversedInvertible, node);
   }
   return tmp.innerHTML;
 }

  /**
   * 渲染向量符号
   */
  KElement.renderVector = function(node){
    var rule = node.getAttribute('data-k-element-rule'),// 渲染规则
      width = rule.split(',')[0],// 待渲染节点的宽
      height = rule.split(',')[1];// 待渲染节点的高
    // 创建canvas
    var underRightArrow = doc.createElement('canvas'),
      context = underRightArrow.getContext('2d');
    underRightArrow.width = width;
    underRightArrow.height = height;
    underRightArrow.setAttribute('data-k-element-type', 'vector');
    underRightArrow.setAttribute('data-k-element-rule', rule);
    underRightArrow.setAttribute('style', node.getAttribute('style'));
    // 绘制右箭头
  	context.beginPath();
    context.lineWidth = 1;
    context.moveTo(0, height/2);
    context.lineTo(width, height/2);
    context.moveTo(width, height/2);
    context.lineTo(width-6, height/2 - 6);
    // context.moveTo(width, height/2);
    // context.lineTo(width-9, height/2 + 6);

    context.stroke();
  	context.closePath();
    // 渲染
    node.parentNode.replaceChild(underRightArrow, node);
  }

 /**
  * 编码向量符号
  */
 KElement.encodeVector = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=vector]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var vector = doc.createElement('k-vector');
     vector.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
     vector.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(vector, node);
   }
   return tmp.innerHTML;
 }

 /**
  * 渲染数学公式
  */
 KElement.renderMath = function(node) {
   // <k-math data-k-element-rule="x_2"></k-math>
   var rule = node.getAttribute('data-k-element-rule');// 渲染规则
   var math = doc.createElement('span');
  //  math.innerHTML = '<span>'+ rule +'</span>';

   math.setAttribute('data-k-element-type', 'math');
   math.setAttribute('data-k-element-rule', rule);
   math.setAttribute('contenteditable', false);
  //  var mathquill = doc.createElement('span');
  //  mathquill.innerText = rule;
  //  math.appendChild(mathquill);
   // 渲染
   node.parentNode.replaceChild(math, node);
  //  new KMathQuill(math.firstChild);
  // try {
  //   KMathQuill.render(rule, math);
  // } catch (error) {}
   KMathJax.ssr(rule, math);
 }

 /**
  * 编码数学公式
  */
 KElement.encodeMath = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('span[data-k-element-type=math]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var math = doc.createElement('k-math');
     math.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
     node.parentNode.replaceChild(math, node);
   }
   return tmp.innerHTML;
 }

 /**
  * 渲染角标
  */
 KElement.renderCorner = function(node){
   console.log('renderCorner', node);
   var outer = doc.createElement('span'),
   cornerL = doc.createElement('span'),
   side = doc.createElement('span'),
   cornerR = doc.createElement('span');

   id = new Date().getTime();
   outer.id = 'k-element-' + id;
   outer.setAttribute('data-k-element-type', 'corner');

   var supL = doc.createElement('span');
   supL.id = 'supL-' + id;
   supL.setAttribute('contenteditable', 'true');
   supL.setAttribute('class', 'sup');
   supL.style.display = 'block';
   supL.style.verticalAlign = 'super';
   supL.style.fontSize = 'smaller';
   supL.innerHTML = '请输入';

   var subL = doc.createElement('span');
   subL.id = 'subL-' + id;
   subL.setAttribute('contenteditable', 'true');
   subL.setAttribute('class', 'sub');
   subL.style.display = 'block';
   subL.style.verticalAlign = 'super';
   subL.style.fontSize = 'smaller';
   subL.innerHTML = '请输入';

   cornerL.style.display = 'inline-block';
   cornerL.style.verticalAlign = 'middle';
   cornerL.appendChild(supL);
   cornerL.appendChild(subL);

   side.innerHTML = '请输入';

   var supR = doc.createElement('span');
   supR.id = 'supR-' + id;
   supR.setAttribute('contenteditable', 'true');
   supR.setAttribute('class', 'sup');
   supR.style.display = 'block';
   supR.style.verticalAlign = 'super';
   supR.style.fontSize = 'smaller';
   supR.innerHTML = '请输入';

   var subR = doc.createElement('span');
   subR.id = 'subR-' + id;
   subR.setAttribute('contenteditable', 'true');
   subR.setAttribute('class', 'sub');
   subR.style.display = 'block';
   subR.style.verticalAlign = 'super';
   subR.style.fontSize = 'smaller';
   subR.innerHTML = '请输入';

   cornerR.style.display = 'inline-block';
   cornerR.style.verticalAlign = 'middle';
   cornerR.appendChild(supR);
   cornerR.appendChild(subR);

   outer.appendChild(cornerL);
   outer.appendChild(side);
   outer.appendChild(cornerR);

   // 渲染
   node.parentNode.replaceChild(underRightArrow, node);
 }

 /**
  * 编码角标
  */
 KElement.encodeCorner = function(html) {
   console.log('encodeCorner', html);
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('span[data-k-element-type=corner]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var vector = doc.createElement('k-vector');
     vector.setAttribute('data-k-element-rule', node.getAttribute('data-k-element-rule'));
     vector.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(vector, node);
   }
   return tmp.innerHTML;
 }

 /**
  * 渲染长等号符
  */
 KElement.renderLongEq = function(node){
   console.log('renderLongEq');
   var width, height;
   width = 42;
   height = 16;

   var canvas = doc.createElement('canvas');
   canvas.id = node.id;
   canvas.width = width;
   canvas.height = height;
   canvas.style.display = 'block';
   canvas.setAttribute('data-k-element-type', 'longEq');

   var context = canvas.getContext("2d");
   context.beginPath();
   context.lineWidth = 1;
   context.moveTo(0, height*0.3);
   context.lineTo(width, height*0.3);
   context.moveTo(0, height*0.7);
   context.lineTo(width, height*0.7);
   context.stroke();
   context.closePath();

   node.parentNode.replaceChild(canvas, node);
 }

 /**
  * 编码长等号符
  */
 KElement.encodeLongEq = function(html) {
   var tmp = doc.createElement('div');
   tmp.innerHTML = html;
   var nodeList = tmp.querySelectorAll('canvas[data-k-element-type=longEq]');
   for (var i = 0; i < nodeList.length; i++) {
     var node = nodeList[i];
     var longEq = doc.createElement('k-long-eq');
     longEq.id = node.id;
     longEq.setAttribute('style', node.getAttribute('style'));
     node.parentNode.replaceChild(longEq, node);
   }
   return tmp.innerHTML;
 }

 /**
  * 渲染平行四边形
  */
 KElement.renderParallelogram = function(node) {
    // 创建canvas
    var parallelogramWrap = doc.createElement('span'),
      parallelogram = doc.createElement('canvas'),
      context = parallelogram.getContext('2d');
    parallelogram.width = 24;
    parallelogram.height = 16;
    // 绘制平行四边形
    context.moveTo(6, 2);
    context.lineTo(24, 2);
    context.lineTo(18, 14);
    context.lineTo(0, 14);
    context.lineTo(6, 2);
    context.stroke();
    parallelogramWrap.setAttribute('data-k-element-type', 'parallelogram');
    parallelogramWrap.setAttribute('contenteditable', false);
    parallelogramWrap.appendChild(parallelogram);
    // 渲染
    node.parentNode.replaceChild(parallelogramWrap, node);
 }

 /**
  * 编码平行四边形
  */
 KElement.encodeParallelogram = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=parallelogram]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var parallelogram = doc.createElement('k-parallelogram');
      node.parentNode.replaceChild(parallelogram, node);
    }
    return tmp.innerHTML;
 }

 /**
  * 渲染方程组
  */
 KElement.renderEquationSet = function(node) {
    // 创建canvas
    var equationSetWarp = doc.createElement('span'),
      equationSetRight = doc.createElement('span'),
      equationSet = doc.createElement('canvas');
    equationSetWarp.setAttribute('style', 'display: inline-block; vertical-align: middle;')
    equationSetRight.setAttribute('style', 'display: inline-block; vertical-align: top;')
    while (node.children.length > 0) {
      equationSetRight.appendChild(node.children[0]);
      if (node.children.length > 0) {
        equationSetRight.appendChild(doc.createElement('br'));
        equationSetRight.appendChild(doc.createElement('br'));
      }
    }
    node.appendChild(equationSetRight);
    var width = 20, height = equationSetRight.offsetHeight;
    // 绘制
    equationSet.width = width;
    equationSet.height = height;
    brace(equationSet);
    equationSetWarp.appendChild(equationSet);
    equationSetWarp.appendChild(equationSetRight);
    equationSetWarp.setAttribute('data-k-element-type', 'equationSet');
    equationSetWarp.setAttribute('contenteditable', false);
    // 渲染
    node.parentNode.replaceChild(equationSetWarp, node);
 }
 /**
  * 编码方程组
  */
 KElement.encodeEquationSet = function(html) {
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=equationSet]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var equationSet = doc.createElement('k-equation-set');
      while (node.children[1].children.length > 0) {
        if (node.children[1].children[0].tagName.toLocaleLowerCase() === 'br') {
          node.children[1].removeChild(node.children[1].children[0]);
          continue;
        }
        equationSet.appendChild(node.children[1].children[0]);
      }
      node.parentNode.replaceChild(equationSet, node);
    }
    return tmp.innerHTML;
 }

 /**
  * 渲染小缩进
  */
 KElement.renderSmallTab = function(node){
   console.log(node);
 }

 /**
  * 编码小缩进
  */
 KElement.encodeSmallTab = function(html){
    var tmp = doc.createElement('div');
    tmp.innerHTML = html;
    var nodeList = tmp.querySelectorAll('span[data-k-element-type=smallTab]');
    for (var i = 0; i < nodeList.length; i++) {
      var node = nodeList[i];
      var smallTab = doc.createElement('k-small-tab');
      node.parentNode.replaceChild(smallTab, node);
    }
    return tmp.innerHTML;
 }

 /**
  * 任务列表
  */
 KElement.taskList = [
   {tagName: 'k-arc', renderFn: KElement.renderArc, encodeFn: KElement.encodeArc},
   {tagName: 'k-underline-gap', renderFn: KElement.renderUnderlineGap, encodeFn: KElement.encodeUnderlineGap},
   {tagName: 'k-block-gap', renderFn: KElement.renderBlockGap, encodeFn: KElement.encodeBlockGap},
   {tagName: 'k-img', renderFn: KElement.renderImg, encodeFn: KElement.encodeImg},
   {tagName: 'k-above-equality', renderFn: KElement.renderAboveEquality, encodeFn: KElement.encodeAboveEquality},
   {tagName: 'k-under-equality', renderFn: KElement.renderUnderEquality, encodeFn: KElement.encodeUnderEquality},
   {tagName: 'k-above-right-arrow', renderFn: KElement.renderAboveRightArrow, encodeFn: KElement.encodeAboveRightArrow},
   {tagName: 'k-under-right-arrow', renderFn: KElement.renderUnderRightArrow, encodeFn: KElement.encodeUnderRightArrow},
   {tagName: 'k-equality', renderFn: KElement.renderEquality, encodeFn: KElement.encodeEquality},
   {tagName: 'k-left-arrow', renderFn: KElement.renderLeftArrow, encodeFn: KElement.encodeLeftArrow},
   {tagName: 'k-right-arrow', renderFn: KElement.renderRightArrow, encodeFn: KElement.encodeRightArrow},
   {tagName: 'k-positive-invertible', renderFn: KElement.renderPositiveInvertible, encodeFn: KElement.encodePositiveInvertible},
   {tagName: 'k-reversed-invertible', renderFn: KElement.renderReversedInvertible, encodeFn: KElement.encodeReversedInvertible},
   {tagName: 'k-vector', renderFn: KElement.renderVector, encodeFn: KElement.encodeVector},
   {tagName: 'k-math', renderFn: KElement.renderMath, encodeFn: KElement.encodeMath},
  //  {tagName: 'k-corner', renderFn: KElement.renderCorner, encodeFn: KElement.encodeCorner},
  {tagName: 'k-long-eq', renderFn: KElement.renderLongEq, encodeFn: KElement.encodeLongEq},
  {tagName: 'k-parallelogram', renderFn: KElement.renderParallelogram, encodeFn: KElement.encodeParallelogram},
  {tagName: 'k-equation-set', renderFn: KElement.renderEquationSet, encodeFn: KElement.encodeEquationSet},
  // {tagName: 'k-small-tab', renderFn: KElement.renderSmallTab, encodeFn: KElement.encodeSmallTab},
 ];

}(window, document));
